import datetime as dt
import re

class DateFormatter(object):

    def __init__(self,start=None,end=None):
        self.start = start
        self.end = end
        return

############################################################################################
#DateTime Formatting
############################################################################################
    def date_formatter(self,dates,delim):
        d = r"[{}]".format(delim)
        if re.search(d, dates) is not None:
            try:
                val = dt.datetime.strptime(str(dates),'%m'+delim+'%d'+delim+'%Y')
                if val is not None:
                    return val
            except ValueError:
                try:
                    val = dt.datetime.strptime(str(dates),'%d'+delim+'%m'+delim+'%Y')
                    if val is not None:
                        return val
                except ValueError:
                    val = dt.datetime.strptime(str(dates),'%Y'+delim+'%m'+delim+'%d')
                    if val is not None:
                        return val
                    else:
                        raise ValueError('Date cannot be formatted')

############################################################################################
#StartDate Formatting
############################################################################################

    def start_date(self, date=None, year=None, month=None, day=None, full=False):
        """ This function defines the start date for the query"""
        try:
            if date is not None:
                #Logic For other Formats
                if self.date_formatter(date,'/') is not None:
                    self.start = self.date_formatter(date,'/')
                    return self.start
                elif self.date_formatter(date,'-') is not None:
                    self.start = self.date_formatter(date,'-')
                    return self.start
                elif self.date_formatter(date,'.') is not None:
                    self.start = self.date_formatter(date,'.')
                    return self.start
                else:
                    raise ValueError('Not a Valid Date. It must be (m/d/y), (d/m/y), or (y/m/d)..')
            elif full is not False:
                self.start = dt.datetime.strptime('1800-01-01','%Y-%m-%d')
                return self.start
            else:
                self.start = dt.datetime(year, month, day)
                return self.start
        except ValueError:
            print("Check the date that you submitted. It must be (m/d/y), (d/m/y), or (y/m/d)..")
############################################################################################
#EndDate Formatting
############################################################################################

    def end_date(self, date=None, year=None, month=None, day=None, full=False):
        """ This function defines the end date for the query."""
        try:
            if date is not None:
                #Logic For other Formats
                if self.date_formatter(date,'/') is not None:
                    self.end = self.date_formatter(date,'/')
                    return self.end
                elif self.date_formatter(date,'-') is not None:
                    self.end = self.date_formatter(date,'-')
                    return self.end
                elif self.date_formatter(date,'.') is not None:
                    self.end = self.date_formatter(date,'.')
                    return self.end
                else:
                    raise ValueError('Not a Valid Date. It must be (m/d/y), (d/m/y), or (y/m/d)..')
            elif full is not False:
                self.end = dt.datetime.now()
                return self.end
            else:
                self.end = dt.datetime(year, month, day)
                return self.end
        except ValueError:
            print("Check the date that you submitted. It must be (m/d/y), (d/m/y), or (y/m/d)..")



import pandas as pd
import pandas_datareader.data as web
import re
import Federal.Formatter.CBSAFormatter as c
import Federal.Formatter.DateFormatter as d

class GDP(object):

    def __init__(self,start=None,end=None):
        self.start = start
        self.end = end
        return

############################################################################################
#Federal: GDP
############################################################################################
    def federal_gdp(self,nominal=False):
        """ This function reads in Real or Nominal GDP in Billions of Dollars.

        nominal (Bool): Defines either Real GDP or Nominal GDP
        """

        code = 'GDP'
        try:
            if nominal is True:
                df = web.DataReader(code+ 'C1', 'fred', self.start, self.end)
                df.rename(columns = {f'{code}C1':'nGDP_2012Chained_Billions$'},inplace=True)
                return df
            else:
                df = web.DataReader(code, 'fred', self.start, self.end)
                df.rename(columns = {f'{code}':'rGDP_2012Chained_Billions$'},inplace=True)
                return df
        except ValueError:
            print('The arguement nominal is a Boolean Value. Please pass True or False.')

############################################################################################
#State: GDP
############################################################################################

    def state_gdp(self, state):
        """ This function creates the correct query to call for a State's rGDP.

        State (String): The 2-Character State ID.
        """

        code = 'NGSP'
        try:
            df = web.DataReader(state + code, 'fred', self.start, self.end)
            df.rename(columns = {f'{state}{code}':f'{state}_GDP_Millions$'}, inplace = True)
            return df
        except ValueError:
             print("Invalid State Abbreviation. The state abbreviation should be a string of length 2")


    def get_metro_name(self,metro_name):
        for k,v in c.CBSA_Codes.items():
            split_metro_list = re.split(r'[-]', v)
            for metro in split_metro_list:
                match = re.search(str(metro_name),metro)
                if match is not None:
                    return k,v
                else:
                    pass

    def get_metro_cbsa_num(self,cbsa_code):
        for k,v in c.CBSA_Codes.items():
            match = re.search(str(cbsa_code),str(k))
            if match is not None:
                return k,v
            else:
                pass

############################################################################################
#Metropolitian: GDP
############################################################################################

    def metro_gdp(self,name=None,cbsa=None,nominal=False):
            """ This function creates the correct query to call for a Metro areas nGDP
            in Million of Chained 2009 Dollars Not Seasonally Adjusted.

            name (String): The metro name that you are searching for.
            cbsa (int):    The CBSA code for a metro.
            nominal (Bool): nominal GDP or Real GDP
            """
            if nominal is True:
                code = 'NGMP'
            else:
                code = 'RGMP'

            if name is not None:
                try:
                    k,v = self.get_metro_name(metro_name=name)
                    df = web.DataReader(code+str(k), 'fred', self.start, self.end)
                    df.rename(columns = {f'{code}{k}':f'{v}_{code}_Millions$'},inplace=True)
                    return df
                except ValueError:
                    print('Not a valid Core-Based Statistical Area (CBSA) city name.')
            elif cbsa is not None:
                try:
                    k,v = self.get_metro_cbsa_num(cbsa_code=cbsa)
                    df = web.DataReader(code+str(k), 'fred', self.start, self.end)
                    df.rename(columns = {f'{code}{k}':f'{v}_{code}_Millions$'},inplace=True)
                    return df
                except ValueError:
                    print('This doesnt look like a valid Core-Based Statistical Area (CBSA), try again.')
            else:
                raise ValueError('Not a valid Core-Based Statistical Area (CBSA) code or name.')



import pandas as pd
import pandas_datareader.data as web
from Federal.Formatter.CBSAFormatter import *
from Federal.Formatter.DateFormatter import *

class Unemployment(object):

    def __init__(self,start=None,end=None):
        self.start = start
        self.end = end
        return

############################################################################################
#Federal: Unemployment Rate
############################################################################################

    def national_unemp(self, sa=True):
        """ This function returns the civilian unemployment rate.

        sa: True if you want sesonally adjusted values, False if not. """

        code = 'UNRATE'
        if sa is True:
            df = web.DataReader(code, 'fred', self.start, self.end)
        else:
            df = web.DataReader(code + 'NSA', 'fred', self.start, self.end)
        return df



# Dictionary of Core-Based Statistical Area (CBSA) Codes
CBSA_Codes = {
    10180 : 'Abilene',
    10380 : 'Aguadilla-Isabela-San Sebastian',
    10420 : 'Akron',
    10500 : 'Albany',
    10580 : 'Albany-Schenectady-Troy',
    10740 : 'Albuquerque',
    10780 : 'Alexandria',
    10900 : 'Allentown-Bethlehem-Easton',
    11020 : 'Altoona',
    11100 : 'Amarillo',
    11180 : 'Ames',
    11260 : 'Anchorage',
    11300 : 'Anderson',
    11340 : 'Anderson',
    11460 : 'Ann Arbor',
    11500 : 'Anniston-Oxford',
    11540 : 'Appleton',
    11700 : 'Asheville',
    12020 : 'Athens-Clarke County',
    12060 : 'Atlanta-Sandy Springs-Marietta',
    12100 : 'Atlantic City-Hammonton',
    12220 : 'Auburn-Opelika',
    12260 : 'Augusta-Richmond County',
    12420 : 'Austin-Round Rock-San Marcos',
    12540 : 'Bakersfield-Delano',
    12580 : 'Baltimore-Towson',
    12620 : 'Bangor',
    12700 : 'Barnstable Town',
    12940 : 'Baton Rouge',
    12980 : 'Battle Creek',
    13020 : 'Bay City',
    13140 : 'Beaumont-Port Arthur',
    13380 : 'Bellingham',
    13460 : 'Bend',
    13740 : 'Billings',
    13780 : 'Binghamton',
    13820 : 'Birmingham-Hoover',
    13900 : 'Bismarck',
    13980 : 'Blacksburg-Christiansburg-Radford',
    14020 : 'Bloomington',
    14060 : 'Bloomington-Normal',
    14260 : 'Boise City-Nampa',
    14460 : 'Boston-Cambridge-Quincy',
    14500 : 'Boulder',
    14540 : 'Bowling Green',
    14740 : 'Bremerton-Silverdale',
    14860 : 'Bridgeport-Stamford-Norwalk',
    15180 : 'Brownsville-Harlingen',
    15260 : 'Brunswick',
    15380 : 'Buffalo-Niagara Falls',
    15500 : 'Burlington',
    15540 : 'Burlington-South Burlington',
    15940 : 'Canton-Massillon',
    15980 : 'Cape Coral-Fort Myers',
    16020 : 'Cape Girardeau-Jackson',
    16180 : 'Carson City',
    16220 : 'Casper',
    16300 : 'Cedar Rapids',
    16580 : 'Champaign-Urbana',
    16620 : 'Charleston',
    16700 : 'Charleston-North Charleston-Summerville',
    16740 : 'Charlotte-Gastonia-Rock Hill',
    16820 : 'Charlottesville',
    16860 : 'Chattanooga',
    16940 : 'Cheyenne',
    16980 : 'Chicago-Joliet-Naperville',
    17020 : 'Chico',
    17140 : 'Cincinnati-Middletown',
    17300 : 'Clarksville',
    17420 : 'Cleveland',
    17460 : 'Cleveland-Elyria-Mentor',
    17660 : "Coeur d'Alene",
    17780 : 'College Station-Bryan',
    17820 : 'Colorado Springs',
    17860 : 'Columbia',
    17900 : 'Columbia',
    17980 : 'Columbus',
    18020 : 'Columbus',
    18140 : 'Columbus',
    18580 : 'Corpus Christi',
    18700 : 'Corvallis',
    18880 : 'Crestview-Fort Walton Beach-Destin',
    19060 : 'Cumberland',
    19100 : 'Dallas-Fort Worth-Arlington',
    19140 : 'Dalton',
    19180 : 'Danville',
    19260 : 'Danville',
    19340 : 'Davenport-Moline-Rock Island',
    19380 : 'Dayton',
    19460 : 'Decatur',
    19500 : 'Decatur',
    19660 : 'Deltona-Daytona Beach-Ormond Beach',
    19740 : 'Denver-Aurora-Broomfield',
    19780 : 'Des Moines-West Des Moines',
    19820 : 'Detroit-Warren-Livonia',
    20020 : 'Dothan',
    20100 : 'Dover',
    20220 : 'Dubuque',
    20260 : 'Duluth',
    20500 : 'Durham-Chapel Hill',
    20740 : 'Eau Claire',
    20940 : 'El Centro',
    21060 : 'Elizabethtown',
    21140 : 'Elkhart-Goshen',
    21300 : 'Elmira',
    21340 : 'El Paso',
    21500 : 'Erie',
    21660 : 'Eugene-Springfield',
    21780 : 'Evansville',
    21820 : 'Fairbanks',
    21940 : 'Fajardo',
    22020 : 'Fargo',
    22140 : 'Farmington',
    22180 : 'Fayetteville',
    22220 : 'Fayetteville-Springdale-Rogers',
    22380 : 'Flagstaff',
    22420 : 'Flint',
    22500 : 'Florence',
    22520 : 'Florence-Muscle Shoals',
    22540 : 'Fond du Lac',
    22660 : 'Fort Collins-Loveland',
    22900 : 'Fort Smith',
    23060 : 'Fort Wayne',
    23420 : 'Fresno',
    23460 : 'Gadsden',
    23540 : 'Gainesville',
    23580 : 'Gainesville',
    24020 : 'Glens Falls',
    24140 : 'Goldsboro',
    24220 : 'Grand Forks',
    24300 : 'Grand Junction',
    24340 : 'Grand Rapids-Wyoming',
    24500 : 'Great Falls',
    24540 : 'Greeley',
    24580 : 'Green Bay',
    24660 : 'Greensboro-High Point',
    24780 : 'Greenville',
    24860 : 'Greenville-Mauldin-Easley',
    25020 : 'Guayama',
    25060 : 'Gulfport-Biloxi',
    25180 : 'Hagerstown-Martinsburg',
    25260 : 'Hanford-Corcoran',
    25420 : 'Harrisburg-Carlisle',
    25500 : 'Harrisonburg',
    25540 : 'Hartford-West Hartford-East Hartford',
    25620 : 'Hattiesburg',
    25860 : 'Hickory-Lenoir-Morganton',
    25980 : 'Hinesville-Fort Stewart',
    26100 : 'Holland-Grand Haven',
    26180 : 'Honolulu',
    26300 : 'Hot Springs',
    26380 : 'Houma-Bayou Cane-Thibodaux',
    26420 : 'Houston-Sugar Land-Baytown',
    26580 : 'Huntington-Ashland',
    26620 : 'Huntsville',
    26820 : 'Idaho Falls',
    26900 : 'Indianapolis-Carmel',
    26980 : 'Iowa City',
    27060 : 'Ithaca',
    27100 : 'Jackson',
    27140 : 'Jackson',
    27180 : 'Jackson',
    27260 : 'Jacksonville',
    27340 : 'Jacksonville',
    27500 : 'Janesville',
    27620 : 'Jefferson City',
    27740 : 'Johnson City',
    27780 : 'Johnstown',
    27860 : 'Jonesboro',
    27900 : 'Joplin',
    28020 : 'Kalamazoo-Portage',
    28100 : 'Kankakee-Bradley',
    28140 : 'Kansas City',
    28420 : 'Kennewick-Pasco-Richland',
    28660 : 'Killeen-Temple-Fort Hood',
    28700 : 'Kingsport-Bristol-Bristol',
    28740 : 'Kingston',
    28940 : 'Knoxville',
    29020 : 'Kokomo',
    29100 : 'La Crosse',
    29140 : 'Lafayette',
    29180 : 'Lafayette',
    29340 : 'Lake Charles',
    29420 : 'Lake Havasu City-Kingman',
    29460 : 'Lakeland-Winter Haven',
    29540 : 'Lancaster',
    29620 : 'Lansing-East Lansing',
    29700 : 'Laredo',
    29740 : 'Las Cruces',
    29820 : 'Las Vegas-Paradise',
    29940 : 'Lawrence',
    30020 : 'Lawton',
    30140 : 'Lebanon',
    30300 : 'Lewiston',
    30340 : 'Lewiston-Auburn',
    30460 : 'Lexington-Fayette',
    30620 : 'Lima',
    30700 : 'Lincoln',
    30780 : 'Little Rock-North Little Rock-Conway',
    30860 : 'Logan',
    30980 : 'Longview',
    31020 : 'Longview',
    31100 : 'Los Angeles-Long Beach-Santa Ana',
    31140 : 'Louisville/Jefferson County',
    31180 : 'Lubbock',
    31340 : 'Lynchburg',
    31420 : 'Macon',
    31460 : 'Madera-Chowchilla',
    31540 : 'Madison',
    31700 : 'Manchester-Nashua',
    31740 : 'Manhattan',
    31860 : 'Mankato-North Mankato',
    31900 : 'Mansfield',
    32420 : 'Mayag�ez',
    32580 : 'McAllen-Edinburg-Mission',
    32780 : 'Medford',
    32820 : 'Memphis',
    32900 : 'Merced',
    33100 : 'Miami-Fort Lauderdale-Pompano Beach',
    33140 : 'Michigan City-La Porte',
    33260 : 'Midland',
    33340 : 'Milwaukee-Waukesha-West Allis',
    33460 : 'Minneapolis-St. Paul-Bloomington',
    33540 : 'Missoula',
    33660 : 'Mobile',
    33700 : 'Modesto',
    33740 : 'Monroe',
    33780 : 'Monroe',
    33860 : 'Montgomery',
    34060 : 'Morgantown',
    34100 : 'Morristown',
    34580 : 'Mount Vernon-Anacortes',
    34620 : 'Muncie',
    34740 : 'Muskegon-Norton Shores',
    34820 : 'Myrtle Beach-North Myrtle Beach-Conway',
    34900 : 'Napa',
    34940 : 'Naples-Marco Island',
    34980 : 'Nashville-Davidson--Murfreesboro--Franklin',
    35300 : 'New Haven-Milford',
    35380 : 'New Orleans-Metairie-Kenner',
    35620 : 'New York-Northern New Jersey-Long Island',
    35660 : 'Niles-Benton Harbor',
    35840 : 'North Port-Bradenton-Sarasota',
    35980 : 'Norwich-New London',
    36100 : 'Ocala',
    36140 : 'Ocean City',
    36220 : 'Odessa',
    36260 : 'Ogden-Clearfield',
    36420 : 'Oklahoma City',
    36500 : 'Olympia',
    36540 : 'Omaha-Council Bluffs',
    36740 : 'Orlando-Kissimmee-Sanford',
    36780 : 'Oshkosh-Neenah',
    36980 : 'Owensboro',
    37100 : 'Oxnard-Thousand Oaks-Ventura',
    37340 : 'Palm Bay-Melbourne-Titusville',
    37380 : 'Palm Coast',
    37460 : 'Panama City-Lynn Haven-Panama City Beach',
    37620 : 'Parkersburg-Marietta-Vienna',
    37700 : 'Pascagoula',
    37860 : 'Pensacola-Ferry Pass-Brent',
    37900 : 'Peoria',
    37980 : 'Philadelphia-Camden-Wilmington',
    38060 : 'Phoenix-Mesa-Glendale',
    38220 : 'Pine Bluff',
    38300 : 'Pittsburgh',
    38340 : 'Pittsfield',
    38540 : 'Pocatello',
    38660 : 'Ponce',
    38860 : 'Portland-South Portland-Biddeford',
    38900 : 'Portland-Vancouver-Hillsboro',
    38940 : 'Port St. Lucie',
    39100 : 'Poughkeepsie-Newburgh-Middletown',
    39140 : 'Prescott',
    39300 : 'Providence-New Bedford-Fall River',
    39340 : 'Provo-Orem',
    39380 : 'Pueblo',
    39460 : 'Punta Gorda',
    39540 : 'Racine',
    39580 : 'Raleigh-Cary',
    39660 : 'Rapid City',
    39740 : 'Reading',
    39820 : 'Redding',
    39900 : 'Reno-Sparks',
    40060 : 'Richmond',
    40140 : 'Riverside-San Bernardino-Ontario',
    40220 : 'Roanoke',
    40340 : 'Rochester',
    40380 : 'Rochester',
    40420 : 'Rockford',
    40580 : 'Rocky Mount',
    40660 : 'Rome',
    40900 : 'Sacramento--Arden-Arcade--Roseville',
    40980 : 'Saginaw-Saginaw Township North',
    41060 : 'St. Cloud',
    41100 : 'St. George',
    41140 : 'St. Joseph',
    41180 : 'St. Louis',
    41420 : 'Salem',
    41500 : 'Salinas',
    41540 : 'Salisbury',
    41620 : 'Salt Lake City',
    41660 : 'San Angelo',
    41700 : 'San Antonio-New Braunfels',
    41740 : 'San Diego-Carlsbad-San Marcos',
    41780 : 'Sandusky',
    41860 : 'San Francisco-Oakland-Fremont',
    41900 : 'San Germ�n-Cabo Rojo',
    41940 : 'San Jose-Sunnyvale-Santa Clara',
    41980 : 'San Juan-Caguas-Guaynabo',
    42020 : 'San Luis Obispo-Paso Robles',
    42060 : 'Santa Barbara-Santa Maria-Goleta',
    42100 : 'Santa Cruz-Watsonville',
    42140 : 'Santa Fe',
    42220 : 'Santa Rosa-Petaluma',
    42340 : 'Savannah',
    42540 : 'Scranton--Wilkes-Barre',
    42660 : 'Seattle-Tacoma-Bellevue',
    42680 : 'Sebastian-Vero Beach',
    43100 : 'Sheboygan',
    43300 : 'Sherman-Denison',
    43340 : 'Shreveport-Bossier City',
    43580 : 'Sioux City',
    43620 : 'Sioux Falls',
    43780 : 'South Bend-Mishawaka',
    43900 : 'Spartanburg',
    44060 : 'Spokane',
    44100 : 'Springfield',
    44140 : 'Springfield',
    44180 : 'Springfield',
    44220 : 'Springfield',
    44300 : 'State College',
    44600 : 'Steubenville-Weirton',
    44700 : 'Stockton',
    44940 : 'Sumter',
    45060 : 'Syracuse',
    45220 : 'Tallahassee',
    45300 : 'Tampa-St. Petersburg-Clearwater',
    45460 : 'Terre Haute',
    45500 : 'Texarkana',
    45780 : 'Toledo',
    45820 : 'Topeka',
    45940 : 'Trenton-Ewing',
    46060 : 'Tucson',
    46140 : 'Tulsa',
    46220 : 'Tuscaloosa',
    46340 : 'Tyler',
    46540 : 'Utica-Rome',
    46660 : 'Valdosta',
    46700 : 'Vallejo-Fairfield',
    47020 : 'Victoria',
    47220 : 'Vineland-Millville-Bridgeton',
    47260 : 'Virginia Beach-Norfolk-Newport News',
    47300 : 'Visalia-Porterville',
    47380 : 'Waco',
    47580 : 'Warner Robins',
    47900 : 'Washington-Arlington-Alexandria',
    47940 : 'Waterloo-Cedar Falls',
    48140 : 'Wausau',
    48300 : 'Wenatchee-East Wenatchee',
    48540 : 'Wheeling',
    48620 : 'Wichita',
    48660 : 'Wichita Falls',
    48700 : 'Williamsport',
    48900 : 'Wilmington',
    49020 : 'Winchester',
    49180 : 'Winston-Salem',
    49340 : 'Worcester',
    49420 : 'Yakima',
    49500 : 'Yauco',
    49620 : 'York-Hanover',
    49660 : 'Youngstown-Warren-Boardman',
    49700 : 'Yuba City',
    49740 : 'Yuma'
}



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

INSTALL_REQUIRES = (
    ['pandas>=0.19.2', 'pandas-datareader=0.7.0', 're', 'datetime']
)

setuptools.setup(
    name='Federal',
    version='1.0.0',
    author='Jeremy A. Seibert',
    author_email='Jaseibert2@eagles.usi.edu',
    description="A wrapper on to the pandas-datareader package for easier handling of federal reserve (FRED) data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Jaseibert/Federal",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
